@@ -1,4 +1,4 @@
-# $Id: core.py 9089 2022-06-22 08:51:16Z milde $
+# $Id: core.py 10267 2025-12-01 22:43:32Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -13,18 +13,26 @@ custom component objects first, and pass *them* to
     https://docutils.sourceforge.io/docs/api/publisher.html
 """
 
+from __future__ import annotations
+
 __docformat__ = 'reStructuredText'
 
+import locale
 import pprint
 import os
 import sys
 import warnings
 
 from docutils import (__version__, __version_details__, SettingsSpec,
-                      io, utils, readers, writers)
+                      io, utils, readers, parsers, writers)
 from docutils.frontend import OptionParser
 from docutils.readers import doctree
 
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    from typing import TextIO
+    from docutils.nodes import StrPath
+
 
 class Publisher:
 
@@ -35,12 +43,25 @@ class Publisher:
     def __init__(self, reader=None, parser=None, writer=None,
                  source=None, source_class=io.FileInput,
                  destination=None, destination_class=io.FileOutput,
-                 settings=None):
+                 settings=None) -> None:
         """
-        Initial setup.  If any of `reader`, `parser`, or `writer` are not
-        specified, the corresponding ``set_...`` method should be called with
-        a component name (`set_reader` sets the parser as well).
+        Initial setup.
+
+        The components `reader`, `parser`, or `writer` should all be
+        specified, either as instances or via their names.
         """
+        # get component instances from their names:
+        if isinstance(reader, str):
+            reader = readers.get_reader_class(reader)(parser)
+        if isinstance(parser, str):
+            if isinstance(reader, readers.Reader):
+                if reader.parser is None:
+                    reader.set_parser(parser)
+                parser = reader.parser
+            else:
+                parser = parsers.get_parser_class(parser)()
+        if isinstance(writer, str):
+            writer = writers.get_writer_class(writer)()
 
         self.document = None
         """The document tree (`docutils.nodes` objects)."""
@@ -54,13 +75,6 @@ class Publisher:
         self.writer = writer
         """A `docutils.writers.Writer` instance."""
 
-        for component in 'reader', 'parser', 'writer':
-            assert not isinstance(getattr(self, component), str), (
-                'passed string "%s" as "%s" parameter; pass an instance, '
-                'or use the "%s_name" parameter instead (in '
-                'docutils.core.publish_* convenience functions).'
-                % (getattr(self, component), component, component))
-
         self.source = source
         """The source of input data, a `docutils.io.Input` instance."""
 
@@ -80,18 +94,29 @@ class Publisher:
 
         self._stderr = io.ErrorOutput()
 
-    def set_reader(self, reader_name, parser, parser_name):
-        """Set `self.reader` by name."""
-        reader_class = readers.get_reader_class(reader_name)
-        self.reader = reader_class(parser, parser_name)
-        self.parser = self.reader.parser
+    def set_reader(self, reader, parser=None, parser_name=None) -> None:
+        """Set `self.reader` by name.
 
-    def set_writer(self, writer_name):
+        The "paser_name" argument is deprecated,
+        use "parser" with parser name or instance.
+        """
+        reader_class = readers.get_reader_class(reader)
+        self.reader = reader_class(parser, parser_name)
+        if self.reader.parser is not None:
+            self.parser = self.reader.parser
+        elif self.parser is not None:
+            self.reader.parser = self.parser
+
+    def set_writer(self, writer_name) -> None:
         """Set `self.writer` by name."""
         writer_class = writers.get_writer_class(writer_name)
         self.writer = writer_class()
 
-    def set_components(self, reader_name, parser_name, writer_name):
+    def set_components(self, reader_name, parser_name, writer_name) -> None:
+        warnings.warn('`Publisher.set_components()` will be removed in '
+                      'Docutils 2.0.  Specify component names '
+                      'at instantiation.',
+                      PendingDeprecationWarning, stacklevel=2)
         if self.reader is None:
             self.set_reader(reader_name, self.parser, parser_name)
         if self.parser is None:
@@ -101,32 +126,27 @@ class Publisher:
         if self.writer is None:
             self.set_writer(writer_name)
 
-    def setup_option_parser(self, usage=None, description=None,
-                            settings_spec=None, config_section=None,
-                            **defaults):
-        warnings.warn('Publisher.setup_option_parser is deprecated, '
-                      'and will be removed in Docutils 0.21.',
-                      DeprecationWarning, stacklevel=2)
-        if config_section:
-            if not settings_spec:
-                settings_spec = SettingsSpec()
-            settings_spec.config_section = config_section
-            parts = config_section.split()
-            if len(parts) > 1 and parts[-1] == 'application':
-                settings_spec.config_section_dependencies = ['applications']
-        # @@@ Add self.source & self.destination to components in future?
-        return OptionParser(
-            components=(self.parser, self.reader, self.writer, settings_spec),
-            defaults=defaults, read_config_files=True,
-            usage=usage, description=description)
-
-    def _setup_settings_parser(self, *args, **kwargs):
+    def _setup_settings_parser(self, usage=None, description=None,
+                               settings_spec=None, config_section=None,
+                               **defaults):
         # Provisional: will change (docutils.frontend.OptionParser will
         # be replaced by a parser based on arparse.ArgumentParser)
         # and may be removed later.
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', category=DeprecationWarning)
-            return self.setup_option_parser(*args, **kwargs)
+            if config_section:
+                if not settings_spec:
+                    settings_spec = SettingsSpec()
+                settings_spec.config_section = config_section
+                parts = config_section.split()
+                if len(parts) > 1 and parts[-1] == 'application':
+                    settings_spec.config_section_dependencies = ['applications']  # noqa: E501
+            # @@@ Add self.source & self.destination to components in future?
+            return OptionParser(
+                components=(self.parser, self.reader, self.writer,
+                            settings_spec),
+                defaults=defaults, read_config_files=True,
+                usage=usage, description=description)
 
     def get_settings(self, usage=None, description=None,
                      settings_spec=None, config_section=None, **defaults):
@@ -147,7 +167,7 @@ class Publisher:
 
     def process_programmatic_settings(self, settings_spec,
                                       settings_overrides,
-                                      config_section):
+                                      config_section) -> None:
         if self.settings is None:
             defaults = settings_overrides.copy() if settings_overrides else {}
             # Propagate exceptions by default when used programmatically:
@@ -158,7 +178,7 @@ class Publisher:
 
     def process_command_line(self, argv=None, usage=None, description=None,
                              settings_spec=None, config_section=None,
-                             **defaults):
+                             **defaults) -> None:
         """
         Parse command line arguments and set ``self.settings``.
 
@@ -173,33 +193,58 @@ class Publisher:
             argv = sys.argv[1:]
         self.settings = option_parser.parse_args(argv)
 
-    def set_io(self, source_path=None, destination_path=None):
+    def set_io(self, source_path=None, destination_path=None) -> None:
         if self.source is None:
             self.set_source(source_path=source_path)
         if self.destination is None:
             self.set_destination(destination_path=destination_path)
 
-    def set_source(self, source=None, source_path=None):
+    def set_source(self,
+                   source: str | None = None,
+                   source_path: StrPath | None = None,
+                   ) -> None:
         if source_path is None:
             source_path = self.settings._source
         else:
+            source_path = os.fspath(source_path)
             self.settings._source = source_path
         self.source = self.source_class(
             source=source, source_path=source_path,
             encoding=self.settings.input_encoding,
             error_handler=self.settings.input_encoding_error_handler)
 
-    def set_destination(self, destination=None, destination_path=None):
-        if destination_path is None:
-            destination_path = self.settings._destination
+    def set_destination(self,
+                        destination: TextIO | None = None,
+                        destination_path: StrPath | None = None,
+                        ) -> None:
+        # Provisional: the "_destination" and "output" settings
+        # are deprecated and will be ignored in Docutils 2.0.
+        if destination_path is not None:
+            self.settings.output_path = os.fspath(destination_path)
         else:
-            self.settings._destination = destination_path
+            # check 'output_path' and legacy settings
+            if getattr(self.settings, 'output', None
+                       ) and not self.settings.output_path:
+                self.settings.output_path = self.settings.output
+            if (self.settings.output_path and self.settings._destination
+                and self.settings.output_path != self.settings._destination):
+                raise SystemExit('The --output-path option obsoletes the '
+                                 'second positional argument (DESTINATION). '
+                                 'You cannot use them together.')
+            if self.settings.output_path is None:
+                self.settings.output_path = self.settings._destination
+            if self.settings.output_path == '-':  # use stdout
+                self.settings.output_path = None
+        self.settings._destination = self.settings.output \
+            = self.settings.output_path
+
         self.destination = self.destination_class(
-            destination=destination, destination_path=destination_path,
+            destination=destination,
+            destination_path=self.settings.output_path,
             encoding=self.settings.output_encoding,
             error_handler=self.settings.output_encoding_error_handler)
 
-    def apply_transforms(self):
+    def apply_transforms(self) -> None:
         self.document.transformer.populate_from_components(
             (self.source, self.reader, self.reader.parser, self.writer,
              self.destination))
@@ -213,7 +258,7 @@ class Publisher:
         already set), run `self.reader` and then `self.writer`.  Return
         `self.writer`'s output.
         """
-        exit = None
+        exit_ = None
         try:
             if self.settings is None:
                 self.process_command_line(
@@ -227,7 +272,7 @@ class Publisher:
             output = self.writer.write(self.document, self.destination)
             self.writer.assemble_parts()
         except SystemExit as error:
-            exit = 1
+            exit_ = True
             exit_status = error.code
         except Exception as error:
             if not self.settings:       # exception too early to report nicely
@@ -236,18 +281,18 @@ class Publisher:
                 self.debugging_dumps()
                 raise
             self.report_Exception(error)
-            exit = True
+            exit_ = True
             exit_status = 1
         self.debugging_dumps()
         if (enable_exit_status and self.document
             and (self.document.reporter.max_level
                  >= self.settings.exit_status_level)):
             sys.exit(self.document.reporter.max_level + 10)
-        elif exit:
+        elif exit_:
             sys.exit(exit_status)
         return output
 
-    def debugging_dumps(self):
+    def debugging_dumps(self) -> None:
         if not self.document:
             return
         if self.settings.dump_settings:
@@ -270,14 +315,14 @@ class Publisher:
             print(self.document.pformat().encode(
                 'raw_unicode_escape'), file=self._stderr)
 
-    def prompt(self):
+    def prompt(self) -> None:
         """Print info and prompt when waiting for input from a terminal."""
         try:
             if not (self.source.isatty() and self._stderr.isatty()):
                 return
         except AttributeError:
             return
-        eot_key = 'Ctrl+Z' if os.name == 'nt' else 'Ctrl+D on an empty line'
+        eot_key = 'Ctrl+Z' if os.name == 'nt' else 'Ctrl+D'
         in_format = ''
         out_format = 'useful formats'
         try:
@@ -288,10 +333,11 @@ class Publisher:
         print(f'Docutils {__version__} <https://docutils.sourceforge.io>\n'
               f'converting "{in_format}" into "{out_format}".\n'
               f'Call with option "--help" for more info.\n'
-              f'.. Waiting for source text (finish with {eot_key}):',
+              f'.. Waiting for source text (finish with {eot_key} '
+              'on an empty line):',
               file=self._stderr)
 
-    def report_Exception(self, error):
+    def report_Exception(self, error) -> None:
         if isinstance(error, utils.SystemMessage):
             self.report_SystemMessage(error)
         elif isinstance(error, UnicodeEncodeError):
@@ -313,12 +359,12 @@ Include "--traceback" output, Docutils version ({__version__}\
 Python version ({sys.version.split()[0]}), your OS type & version, \
 and the command line used.""", file=self._stderr)
 
-    def report_SystemMessage(self, error):
+    def report_SystemMessage(self, error) -> None:
         print('Exiting due to level-%s (%s) system message.' % (
                   error.level, utils.Reporter.levels[error.level]),
               file=self._stderr)
 
-    def report_UnicodeError(self, error):
+    def report_UnicodeError(self, error) -> None:
         data = error.object[error.start:error.end]
         self._stderr.write(
             '%s\n'
@@ -360,24 +406,25 @@ default_description = (
 # TODO: or not to do?  cf. https://clig.dev/#help
 #
 # Display output on success, but keep it brief.
-#
 # Provide a -q option to suppress all non-essential output.
+#
 # Chain several args as input and use --output or redirection for output:
 #   argparser.add_argument('source', nargs='+')
 #
-def publish_cmdline(reader=None, reader_name='standalone',
-                    parser=None, parser_name='restructuredtext',
-                    writer=None, writer_name='pseudoxml',
+def publish_cmdline(reader=None, reader_name=None,
+                    parser=None, parser_name=None,
+                    writer=None, writer_name=None,
                     settings=None, settings_spec=None,
                     settings_overrides=None, config_section=None,
                     enable_exit_status=True, argv=None,
                     usage=default_usage, description=default_description):
     """
     Set up & run a `Publisher` for command-line-based file I/O (input and
-    output file paths taken automatically from the command line).  Return the
-    encoded string output also.
+    output file paths taken automatically from the command line).
+    Also return the output as `str` or `bytes` (for binary output document
+    formats).
 
-    Parameters: see `publish_programmatically` for the remainder.
+    Parameters: see `publish_programmatically()` for the remainder.
 
     - `argv`: Command-line argument list to use instead of ``sys.argv[1:]``.
     - `usage`: Usage string, output if there's a problem parsing the command
@@ -385,9 +432,14 @@ def publish_cmdline(reader=None, reader_name='standalone',
     - `description`: Program description, output for the "--help" option
       (along with command-line option descriptions).
     """
-    pub = Publisher(reader, parser, writer, settings=settings)
-    pub.set_components(reader_name, parser_name, writer_name)
-    output = pub.publish(
+    # The "*_name" arguments are deprecated.
+    _name_arg_warning(reader_name, parser_name, writer_name)
+    # The default is only used if both arguments are empty
+    reader = reader or reader_name or 'standalone'
+    parser = parser or parser_name or 'restructuredtext'
+    writer = writer or writer_name or 'pseudoxml'
+    publisher = Publisher(reader, parser, writer, settings=settings)
+    output = publisher.publish(
         argv, usage, description, settings_spec, settings_overrides,
         config_section=config_section, enable_exit_status=enable_exit_status)
     return output
@@ -395,18 +447,22 @@ def publish_cmdline(reader=None, reader_name='standalone',
 
 def publish_file(source=None, source_path=None,
                  destination=None, destination_path=None,
-                 reader=None, reader_name='standalone',
-                 parser=None, parser_name='restructuredtext',
-                 writer=None, writer_name='pseudoxml',
+                 reader=None, reader_name=None,
+                 parser=None, parser_name=None,
+                 writer=None, writer_name=None,
                  settings=None, settings_spec=None, settings_overrides=None,
                  config_section=None, enable_exit_status=False):
     """
     Set up & run a `Publisher` for programmatic use with file-like I/O.
-    Return the encoded string output also.
+    Also return the output as `str` or `bytes` (for binary output document
+    formats).
 
-    Parameters: see `publish_programmatically`.
+    Parameters: see `publish_programmatically()`.
     """
-    output, pub = publish_programmatically(
+    # The "*_name" arguments are deprecated.
+    _name_arg_warning(reader_name, parser_name, writer_name)
+    # The default is set in publish_programmatically().
+    output, _publisher = publish_programmatically(
         source_class=io.FileInput, source=source, source_path=source_path,
         destination_class=io.FileOutput,
         destination=destination, destination_path=destination_path,
@@ -421,29 +477,34 @@ def publish_file(source=None, source_path=None,
 
 
 def publish_string(source, source_path=None, destination_path=None,
-                   reader=None, reader_name='standalone',
-                   parser=None, parser_name='restructuredtext',
-                   writer=None, writer_name='pseudoxml',
+                   reader=None, reader_name=None,
+                   parser=None, parser_name=None,
+                   writer=None, writer_name=None,
                    settings=None, settings_spec=None,
                    settings_overrides=None, config_section=None,
                    enable_exit_status=False):
     """
-    Set up & run a `Publisher` for programmatic use with string I/O.  Return
-    the encoded string or Unicode string output.
+    Set up & run a `Publisher` for programmatic use with string I/O.
 
-    For encoded string output, be sure to set the 'output_encoding' setting to
-    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
-    output.  Here's one way::
+    Accepts a `bytes` or `str` instance as `source`.
 
-        publish_string(..., settings_overrides={'output_encoding': 'unicode'})
+    The output is encoded according to the `output_encoding`_ setting;
+    the return value is a `bytes` instance (unless `output_encoding`_ is
+    "unicode", cf. `docutils.io.StringOutput.write()`).
 
-    Similarly for Unicode string input (`source`)::
+    Parameters: see `publish_programmatically()` or
+    https://docutils.sourceforge.io/docs/api/publisher.html#publish-string
 
-        publish_string(..., settings_overrides={'input_encoding': 'unicode'})
+    This function is provisional because in Python 3 name and behaviour
+    no longer match.
 
-    Parameters: see `publish_programmatically`.
+    .. _output_encoding:
+        https://docutils.sourceforge.io/docs/user/config.html#output-encoding
     """
-    output, pub = publish_programmatically(
+    # The "*_name" arguments are deprecated.
+    _name_arg_warning(reader_name, parser_name, writer_name)
+    # The default is set in publish_programmatically().
+    output, _publisher = publish_programmatically(
         source_class=io.StringInput, source=source, source_path=source_path,
         destination_class=io.StringOutput,
         destination=None, destination_path=destination_path,
@@ -459,26 +520,32 @@ def publish_string(source, source_path=None, destination_path=None,
 
 def publish_parts(source, source_path=None, source_class=io.StringInput,
                   destination_path=None,
-                  reader=None, reader_name='standalone',
-                  parser=None, parser_name='restructuredtext',
-                  writer=None, writer_name='pseudoxml',
+                  reader=None, reader_name=None,
+                  parser=None, parser_name=None,
+                  writer=None, writer_name=None,
                   settings=None, settings_spec=None,
                   settings_overrides=None, config_section=None,
                   enable_exit_status=False):
     """
     Set up & run a `Publisher`, and return a dictionary of document parts.
-    Dictionary keys are the names of parts, and values are Unicode strings;
-    encoding is up to the client.  For programmatic use with string I/O.
 
-    For encoded string input, be sure to set the 'input_encoding' setting to
-    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
-    input.  Here's how::
+    Dictionary keys are the names of parts.
+    Dictionary values are `str` instances; encoding is up to the client,
+    e.g.::
 
-        publish_parts(..., settings_overrides={'input_encoding': 'unicode'})
+       parts = publish_parts(...)
+       body = parts['body'].encode(parts['encoding'], parts['errors'])
 
-    Parameters: see `publish_programmatically`.
+    See the `API documentation`__ for details on the provided parts.
+
+    Parameters: see `publish_programmatically()`.
+
+    __ https://docutils.sourceforge.io/docs/api/publisher.html#publish-parts
     """
-    output, pub = publish_programmatically(
+    # The "*_name" arguments are deprecated.
+    _name_arg_warning(reader_name, parser_name, writer_name)
+    # The default is set in publish_programmatically().
+    _output, publisher = publish_programmatically(
         source=source, source_path=source_path, source_class=source_class,
         destination_class=io.StringOutput,
         destination=None, destination_path=destination_path,
@@ -489,79 +556,73 @@ def publish_parts(source, source_path=None, source_class=io.StringInput,
         settings_overrides=settings_overrides,
         config_section=config_section,
         enable_exit_status=enable_exit_status)
-    return pub.writer.parts
+    return publisher.writer.parts
 
 
 def publish_doctree(source, source_path=None,
                     source_class=io.StringInput,
-                    reader=None, reader_name='standalone',
-                    parser=None, parser_name='restructuredtext',
+                    reader=None, reader_name=None,
+                    parser=None, parser_name=None,
                     settings=None, settings_spec=None,
                     settings_overrides=None, config_section=None,
                     enable_exit_status=False):
     """
-    Set up & run a `Publisher` for programmatic use with string I/O.
-    Return the document tree.
+    Set up & run a `Publisher` for programmatic use. Return a document tree.
 
-    For encoded string input, be sure to set the 'input_encoding' setting to
-    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
-    input.  Here's one way::
-
-        publish_doctree(..., settings_overrides={'input_encoding': 'unicode'})
-
-    Parameters: see `publish_programmatically`.
+    Parameters: see `publish_programmatically()`.
     """
-    pub = Publisher(reader=reader, parser=parser, writer=None,
-                    settings=settings,
-                    source_class=source_class,
-                    destination_class=io.NullOutput)
-    pub.set_components(reader_name, parser_name, 'null')
-    pub.process_programmatic_settings(
-        settings_spec, settings_overrides, config_section)
-    pub.set_source(source, source_path)
-    pub.set_destination(None, None)
-    pub.publish(enable_exit_status=enable_exit_status)
-    return pub.document
+    # The "*_name" arguments are deprecated.
+    _name_arg_warning(reader_name, parser_name, None)
+    # The default is set in publish_programmatically().
+    _output, publisher = publish_programmatically(
+        source=source, source_path=source_path,
+        source_class=source_class,
+        destination=None, destination_path=None,
+        destination_class=io.NullOutput,
+        reader=reader, reader_name=reader_name,
+        parser=parser, parser_name=parser_name,
+        writer='null', writer_name=None,
+        settings=settings, settings_spec=settings_spec,
+        settings_overrides=settings_overrides, config_section=config_section,
+        enable_exit_status=enable_exit_status)
+    return publisher.document
 
 
 def publish_from_doctree(document, destination_path=None,
-                         writer=None, writer_name='pseudoxml',
+                         writer=None, writer_name=None,
                          settings=None, settings_spec=None,
                          settings_overrides=None, config_section=None,
                          enable_exit_status=False):
     """
-    Set up & run a `Publisher` to render from an existing document
-    tree data structure, for programmatic use with string I/O.  Return
-    the encoded string output.
+    Set up & run a `Publisher` to render from an existing document tree
+    data structure. For programmatic use with string output
+    (`bytes` or `str`, cf. `publish_string()`).
 
-    Note that document.settings is overridden; if you want to use the settings
-    of the original `document`, pass settings=document.settings.
+    Note that ``document.settings`` is overridden; if you want to use the
+    settings of the original `document`, pass ``settings=document.settings``.
 
-    Also, new document.transformer and document.reporter objects are
+    Also, new `document.transformer` and `document.reporter` objects are
     generated.
 
-    For encoded string output, be sure to set the 'output_encoding' setting to
-    the desired encoding.  Set it to 'unicode' for unencoded Unicode string
-    output.  Here's one way::
-
-        publish_from_doctree(
-            ..., settings_overrides={'output_encoding': 'unicode'})
-
     Parameters: `document` is a `docutils.nodes.document` object, an existing
     document tree.
 
-    Other parameters: see `publish_programmatically`.
+    Other parameters: see `publish_programmatically()`.
+
+    This function is provisional because in Python 3 name and behaviour
+    of the `io.StringOutput` class no longer match.
     """
-    reader = doctree.Reader(parser_name='null')
-    pub = Publisher(reader, None, writer,
-                    source=io.DocTreeInput(document),
-                    destination_class=io.StringOutput, settings=settings)
-    if not writer and writer_name:
-        pub.set_writer(writer_name)
-    pub.process_programmatic_settings(
+    # The "writer_name" argument is deprecated.
+    _name_arg_warning(None, None, writer_name)
+    publisher = Publisher(reader=doctree.Reader(),
+                          writer=writer or writer_name or 'pseudoxml',
+                          source=io.DocTreeInput(document),
+                          destination_class=io.StringOutput,
+                          settings=settings)
+    publisher.process_programmatic_settings(
         settings_spec, settings_overrides, config_section)
-    pub.set_destination(None, destination_path)
-    return pub.publish(enable_exit_status=enable_exit_status)
+    publisher.set_destination(None, destination_path)
+    return publisher.publish(enable_exit_status=enable_exit_status)
 
 
 def publish_cmdline_to_binary(reader=None, reader_name='standalone',
@@ -579,29 +640,44 @@ def publish_cmdline_to_binary(reader=None, reader_name='standalone',
                               destination_class=io.BinaryFileOutput):
     """
     Set up & run a `Publisher` for command-line-based file I/O (input and
-    output file paths taken automatically from the command line).  Return the
-    encoded string output also.
+    output file paths taken automatically from the command line).
+    Also return the output as `bytes`.
 
     This is just like publish_cmdline, except that it uses
     io.BinaryFileOutput instead of io.FileOutput.
 
-    Parameters: see `publish_programmatically` for the remainder.
+    Parameters: see `publish_programmatically()` for the remainder.
 
     - `argv`: Command-line argument list to use instead of ``sys.argv[1:]``.
     - `usage`: Usage string, output if there's a problem parsing the command
       line.
     - `description`: Program description, output for the "--help" option
       (along with command-line option descriptions).
+
+    Deprecated. Use `publish_cmdline()` (works with `bytes` since
+    Docutils 0.20). Will be removed in Docutils 0.24.
     """
-    pub = Publisher(reader, parser, writer, settings=settings,
-                    destination_class=destination_class)
-    pub.set_components(reader_name, parser_name, writer_name)
-    output = pub.publish(
+    warnings.warn('"publish_cmdline_to_binary()" is obsoleted'
+                  ' by "publish_cmdline()" and will be removed'
+                  ' in Docutils 0.24.', DeprecationWarning, stacklevel=2)
+    publisher = Publisher(reader, parser, writer, settings=settings,
+                          destination_class=destination_class)
+    publisher.set_components(reader_name, parser_name, writer_name)
+    output = publisher.publish(
         argv, usage, description, settings_spec, settings_overrides,
         config_section=config_section, enable_exit_status=enable_exit_status)
     return output
 
 
+def _name_arg_warning(*name_args) -> None:
+    for component, name_arg in zip(('reader', 'parser', 'writer'), name_args):
+        if name_arg is not None:
+            warnings.warn(f'Argument "{component}_name" will be removed in '
+                          f'Docutils 2.0.  Specify {component} name '
+                          f'in the "{component}" argument.',
+                          PendingDeprecationWarning, stacklevel=3)
+
+
 def publish_programmatically(source_class, source, source_path,
                              destination_class, destination, destination_path,
                              reader, reader_name,
@@ -611,10 +687,13 @@ def publish_programmatically(source_class, source, source_path,
                              settings_overrides, config_section,
                              enable_exit_status):
     """
-    Set up & run a `Publisher` for custom programmatic use.  Return the
-    encoded string output and the Publisher object.
+    Set up & run a `Publisher` for custom programmatic use.
 
-    Applications should not need to call this function directly.  If it does
+    Return the output (as `str` or `bytes`, depending on `destination_class`,
+    writer, and the "output_encoding" setting) and the Publisher object.
+
+    Internal:
+    Applications should not call this function directly.  If it does
     seem to be necessary to call this function directly, please write to the
     Docutils-develop mailing list
     <https://docutils.sourceforge.io/docs/user/mailing-lists.html#docutils-develop>.
@@ -631,18 +710,17 @@ def publish_programmatically(source_class, source, source_path,
         (`source_path` is opened).  If neither `source` nor
         `source_path` are supplied, `sys.stdin` is used.
 
-      - If `source_class` is `io.StringInput` **required**: The input
-        string, either an encoded 8-bit string (set the
-        'input_encoding' setting to the correct encoding) or a Unicode
-        string (set the 'input_encoding' setting to 'unicode').
+      - If `source_class` is `io.StringInput` **required**:
+        The input as either a `bytes` object (ensure the 'input_encoding'
+        setting matches its encoding) or a `str` object.
 
     * `source_path`: Type depends on `source_class`:
 
       - `io.FileInput`: Path to the input file, opened if no `source`
         supplied.
 
-      - `io.StringInput`: Optional.  Path to the file or object that produced
-        `source`.  Only used for diagnostic output.
+      - `io.StringInput`: Optional.  Path to the file or description of the
+        object that produced `source`.  Only used for diagnostic output.
 
     * `destination_class` **required**: The class for dynamically created
       destination objects.  Typically `io.FileOutput` or `io.StringOutput`.
@@ -665,20 +743,20 @@ def publish_programmatically(source_class, source, source_path,
         output; optional.  Used for determining relative paths (stylesheets,
         source links, etc.).
 
-    * `reader`: A `docutils.readers.Reader` object.
+    * `reader`: A `docutils.readers.Reader` instance, name, or alias.
+      Default: "standalone".
 
-    * `reader_name`: Name or alias of the Reader class to be instantiated if
-      no `reader` supplied.
+    * `reader_name`: Deprecated. Use `reader`.
 
-    * `parser`: A `docutils.parsers.Parser` object.
+    * `parser`: A `docutils.parsers.Parser` instance, name, or alias.
+      Default: "restructuredtext".
 
-    * `parser_name`: Name or alias of the Parser class to be instantiated if
-      no `parser` supplied.
+    * `parser_name`: Deprecated. Use `parser`.
 
-    * `writer`: A `docutils.writers.Writer` object.
+    * `writer`: A `docutils.writers.Writer` instance, name, or alias.
+      Default: "pseudoxml".
 
-    * `writer_name`: Name or alias of the Writer class to be instantiated if
-      no `writer` supplied.
+    * `writer_name`: Deprecated. Use `writer`.
 
     * `settings`: A runtime settings (`docutils.frontend.Values`) object, for
       dotted-attribute access to runtime settings.  It's the end result of the
@@ -702,13 +780,76 @@ def publish_programmatically(source_class, source, source_path,
 
     * `enable_exit_status`: Boolean; enable exit status at end of processing?
     """
-    pub = Publisher(reader, parser, writer, settings=settings,
-                    source_class=source_class,
-                    destination_class=destination_class)
-    pub.set_components(reader_name, parser_name, writer_name)
-    pub.process_programmatic_settings(
+    reader = reader or reader_name or 'standalone'
+    parser = parser or parser_name or 'restructuredtext'
+    writer = writer or writer_name or 'pseudoxml'
+
+    publisher = Publisher(reader, parser, writer, settings=settings,
+                          source_class=source_class,
+                          destination_class=destination_class)
+    publisher.process_programmatic_settings(
         settings_spec, settings_overrides, config_section)
-    pub.set_source(source, source_path)
-    pub.set_destination(destination, destination_path)
-    output = pub.publish(enable_exit_status=enable_exit_status)
-    return output, pub
+    publisher.set_source(source, source_path)
+    publisher.set_destination(destination, destination_path)
+    output = publisher.publish(enable_exit_status=enable_exit_status)
+    return output, publisher
+
+
+# "Entry points" with functionality of the "tools/rst2*.py" scripts
+# cf. https://packaging.python.org/en/latest/specifications/entry-points/
+
+def rst2something(writer, documenttype, doc_path='') -> None:
+    # Helper function for the common parts of `rst2*()`
+    #   writer:       writer name
+    #   documenttype: output document type
+    #   doc_path:     documentation path (relative to the documentation root)
+    description = (
+        f'Generate {documenttype} documents '
+        'from standalone reStructuredText sources '
+        f'<https://docutils.sourceforge.io/docs/{doc_path}>.  '
+        + default_description)
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+    except locale.Error as e:
+        sys.stderr.write(f'WARNING: Cannot set the default locale: {e}.\n')
+    publish_cmdline(writer=writer, description=description)
+
+
+def rst2html() -> None:
+    rst2something('html', 'HTML', 'user/html.html#html')
+
+
+def rst2html4() -> None:
+    rst2something('html4', 'XHTML 1.1', 'user/html.html#html4css1')
+
+
+def rst2html5() -> None:
+    rst2something('html5', 'HTML5', 'user/html.html#html5-polyglot')
+
+
+def rst2latex() -> None:
+    rst2something('latex', 'LaTeX', 'user/latex.html')
+
+
+def rst2man() -> None:
+    rst2something('manpage', 'Unix manual (troff)', 'user/manpage.html')
+
+
+def rst2odt() -> None:
+    rst2something('odt', 'OpenDocument text (ODT)', 'user/odt.html')
+
+
+def rst2pseudoxml() -> None:
+    rst2something('pseudoxml', 'pseudo-XML (test)', 'ref/doctree.html')
+
+
+def rst2s5() -> None:
+    rst2something('s5', 'S5 HTML slideshow', 'user/slide-shows.html')
+
+
+def rst2xetex() -> None:
+    rst2something('xetex', 'LaTeX (XeLaTeX/LuaLaTeX)', 'user/latex.html')
+
+
+def rst2xml() -> None:
+    rst2something('xml', 'Docutils-native XML', 'ref/doctree.html')
