@@ -1,9 +1,9 @@
 # Source and destination file names
-test_source = "data/math.txt"
+test_source = "data/math.rst"
 test_destination = "math_output_mathml.html"
 
 # Keyword parameters passed to publish_file()
-writer_name = "html5"
+writer = "html5"
 settings_overrides = {
     'math_output': 'MathML',
     # location of stylesheets (relative to ``docutils/test/``)
