@@ -1,9 +1,9 @@
 # Source and destination file names
-test_source = "standalone_rst_xetex.txt"
+test_source = "standalone_rst_xetex.rst"
 test_destination = "standalone_rst_xetex.tex"
 
 # Keyword parameters passed to publish_file()
-writer_name = "xetex"
+writer = "xetex"
 settings_overrides = {
     'sectsubtitle_xform': True,
     # use "smartquotes" transition:
