@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 
-# $Id: test_code.py 9425 2023-06-30 14:56:47Z milde $
+# $Id: test_code.py 10134 2025-05-19 21:12:34Z milde $
 # Author: Guenter Milde
 # Copyright: This module has been placed in the public domain.
 
@@ -9,7 +9,6 @@ Test the 'code' directive in parsers/rst/directives/body.py.
 """
 
 from pathlib import Path
-import re
 import sys
 import unittest
 
@@ -25,17 +24,20 @@ from docutils.utils.code_analyzer import with_pygments
 
 if with_pygments:
     import pygments
-    _pv = re.match(r'^([0-9]+)\.([0-9]*)', pygments.__version__)
-    PYGMENTS_2_14_PLUS = (int(_pv[1]), int(_pv[2])) >= (2, 14)
+
+    pygments_version = tuple(map(int, pygments.__version__.split('.')[:2]))
 else:
-    PYGMENTS_2_14_PLUS = None
+    pygments_version = (0, 0)
+
+PYGMENTS_2_14_PLUS = pygments_version >= (2, 14)
+if pygments_version >= (2, 19):
+    def_ws = '<inline classes="whitespace">\n             '
+else:
+    def_ws = ' '
 
 
 class ParserTestCase(unittest.TestCase):
     def test_parser(self):
-        if not with_pygments:
-            del totest['code_parsing']
-
         parser = Parser()
         settings = get_default_settings(Parser)
         settings.warning_stream = ''
@@ -43,6 +45,8 @@ class ParserTestCase(unittest.TestCase):
         for name, cases in totest.items():
             for casenum, (case_input, case_expected) in enumerate(cases):
                 with self.subTest(id=f'totest[{name!r}][{casenum}]'):
+                    if name == 'code_parsing' and not with_pygments:
+                        self.skipTest('syntax highlight requires pygments')
                     document = new_document('test data', settings.copy())
                     parser.parse(case_input, document)
                     output = document.pformat()
@@ -160,14 +164,14 @@ totest['code_parsing'] = [
       # and now for something completely different
       print(8/2)
 """,
-"""\
+f"""\
 <document source="test data">
     <literal_block classes="code python3 testclass" ids="my-function" names="my_function" xml:space="preserve">
         <inline classes="ln">
              7 \n\
         <inline classes="keyword">
             def
-         \n\
+        {def_ws}
         <inline classes="name function">
             my_function
         <inline classes="punctuation">
