@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 
-# $Id: test_contents.py 9037 2022-03-05 23:31:10Z milde $
+# $Id: test_contents.py 9425 2023-06-30 14:56:47Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -8,17 +8,41 @@
 Tests for parts.py contents directive.
 """
 
+from pathlib import Path
+import sys
+import unittest
+
 if __name__ == '__main__':
-    import __init__  # noqa: F401
-from test_parsers import DocutilsTestSupport
+    # prepend the "docutils root" to the Python library path
+    # so we import the local `docutils` package.
+    sys.path.insert(0, str(Path(__file__).resolve().parents[4]))
+
+from docutils.frontend import get_default_settings
+from docutils.parsers.rst import Parser
+from docutils.utils import new_document
 
 
-def suite():
-    s = DocutilsTestSupport.ParserTestSuite()
-    s.generateTests(totest)
-    return s
+class ParserTestCase(unittest.TestCase):
+    def test_parser(self):
+        parser = Parser()
+        settings = get_default_settings(Parser)
+        settings.warning_stream = ''
+        for name, cases in totest.items():
+            for casenum, (case_input, case_expected) in enumerate(cases):
+                with self.subTest(id=f'totest[{name!r}][{casenum}]'):
+                    document = new_document('test data', settings.copy())
+                    parser.parse(case_input, document)
+                    output = document.pformat()
+                    self.assertEqual(case_expected, output)
 
 
+try:
+    int('two')
+except ValueError as detail:
+    invalid_literal = detail.args[0]
+else:
+    invalid_literal = ''
+
 totest = {}
 
 totest['contents'] = [
@@ -159,7 +183,7 @@ totest['contents'] = [
         <literal_block xml:space="preserve">
             .. contents::
                :depth: two
-""" % DocutilsTestSupport.exception_data(int, "two")[1][0]],
+""" % invalid_literal],
 ["""\
 .. contents::
    :width: 2
@@ -239,5 +263,4 @@ totest['contents'] = [
 
 
 if __name__ == '__main__':
-    import unittest
-    unittest.main(defaultTest='suite')
+    unittest.main()
