@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 
-# $Id: test_viewlist.py 9277 2022-11-26 23:15:13Z milde $
+# $Id: test_viewlist.py 9839 2024-08-01 21:25:56Z aa-turner $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -46,7 +46,7 @@ class ViewListTests(unittest.TestCase):
         self.assertEqual(self.c, self.c_list)
         self.assertEqual(len(self.a), len(self.a_list))
         self.assertTrue('d' in self.a)  # __contains__
-        self.assertEqual([value for value in self.a], self.a_list)
+        self.assertEqual(list(self.a), self.a_list)
         # get and set values
         self.assertEqual(self.a[2], self.a_list[2])
         a = self.a[:]
